import time
from typing import Callable
from fastapi import Request
from starlette.middleware.base import BaseHTTPMiddleware

from nlbone.config.logging import get_logger

logger = get_logger(__name__)

class AccessLogMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next: Callable):
        start = time.perf_counter()
        status_code = None
        try:
            response = await call_next(request)
            status_code = getattr(response, "status_code", None)
            return response
        except Exception:
            status_code = 500
            raise
        finally:
            dur_ms = int((time.perf_counter() - start) * 1000)
            logger.info({
                "event": "access",
                "method": request.method,
                "path": request.url.path,
                "status": status_code,
                "duration_ms": dur_ms,
                "query": request.url.query,
            })



from typing import Optional, Callable, Union
from fastapi import Request
from starlette.middleware.base import BaseHTTPMiddleware

try:
    from dependency_injector import providers
    ProviderType = providers.Provider  # type: ignore
except Exception:
    ProviderType = object

from nlbone.core.ports.auth import AuthService


def _to_factory(auth: Union[AuthService, Callable[[], AuthService], ProviderType]):
    try:
        from dependency_injector import providers as _p  # type: ignore
        if isinstance(auth, _p.Provider):
            return auth
    except Exception:
        pass
    if callable(auth) and not hasattr(auth, "verify_token"):
        return auth
    return lambda: auth


class AuthenticationMiddleware(BaseHTTPMiddleware):
    def __init__(self, app, auth: Union[AuthService, Callable[[], AuthService], ProviderType]):
        super().__init__(app)
        self._get_auth = _to_factory(auth)

    async def dispatch(self, request: Request, call_next):
        request.state.client_id = request.headers.get("X-Client-Id")
        request.state.user_id = None
        request.state.token = None

        token: Optional[str] = None
        authz = request.headers.get("Authorization")
        if authz:
            try:
                scheme, token = authz.split(" ", 1)
                if scheme.lower() != "bearer":
                    token = None
            except ValueError:
                token = None

        if token:
            request.state.token = token
            try:
                service: AuthService = self._get_auth()
                data = service.verify_token(token)
                if data:
                    request.state.user_id = data.get("user_id")
            except Exception:
                pass

        return await call_next(request)



from sqlalchemy.sql.sqltypes import (
    String, Text, Integer, BigInteger, SmallInteger, Numeric, Float, Boolean, Enum as SAEnum
)
try:
    from sqlalchemy.dialects.postgresql import ENUM as PGEnum
except Exception:
    PGEnum = type("PGEnum", (), {})

from .types import InvalidEnum

def is_text_type(coltype) -> bool:
    return isinstance(coltype, (String, Text))

def looks_like_wildcard(s: str) -> bool:
    return isinstance(s, str) and ("*" in s or "%" in s)

def to_like_pattern(s: str) -> str:
    s = (s or "")
    s = s.replace("*", "%")
    return s if "%" in s else f"%{s}%"

def _coerce_enum(col_type, raw):
    if raw is None:
        return None
    enum_cls = getattr(col_type, "enum_class", None)
    if enum_cls is not None:
        if isinstance(raw, enum_cls):
            return raw
        if isinstance(raw, str):
            low = raw.strip().lower()
            for m in enum_cls:
                if m.name.lower() == low or str(m.value).lower() == low:
                    return m
        raise InvalidEnum(f"'{raw}' is not one of {[m.name for m in enum_cls]}")
    choices = list(getattr(col_type, "enums", []) or [])
    if isinstance(raw, str):
        low = raw.strip().lower()
        for c in choices:
            if c.lower() == low:
                return c
    raise InvalidEnum(f"'{raw}' is not one of {choices or '[no choices defined]'}")

def coerce_value_for_column(coltype, v):
    if v is None:
        return None
    if isinstance(coltype, (SAEnum, PGEnum)):
        return _coerce_enum(coltype, v)
    if is_text_type(coltype):
        return str(v)
    if isinstance(coltype, (Integer, BigInteger, SmallInteger)):
        return int(v)
    if isinstance(coltype, (Float, Numeric)):
        return float(v)
    if isinstance(coltype, Boolean):
        if isinstance(v, bool):
            return v
        if isinstance(v, (int, float)):
            return bool(v)
        if isinstance(v, str):
            vl = v.strip().lower()
            if vl in {"true", "1", "yes", "y", "t"}: return True
            if vl in {"false", "0", "no", "n", "f"}: return False
        return None
    return v



from __future__ import annotations
from sqlalchemy import or_
from sqlalchemy.orm import Query

try:
    from sqlalchemy.dialects.postgresql import ENUM as PGEnum  # اختیاری
except Exception:
    PGEnum = type("PGEnum", (), {})  # fallback dummy

from nlbone.interfaces.api.exceptions import UnprocessableEntityException
from nlbone.interfaces.api.pagination import PaginateRequest
from .coercion import coerce_value_for_column, to_like_pattern, is_text_type, looks_like_wildcard
from .types import NULL_SENTINELS, InvalidEnum, parse_field_and_op

def apply_filters_to_query(pagination: PaginateRequest, entity, query: Query) -> Query:
    if not getattr(pagination, "filters", None):
        return query

    for raw_field, value in pagination.filters.items():
        if value is None or value in NULL_SENTINELS or value == [] or value == {}:
            value = None

        field, op_hint = parse_field_and_op(raw_field)
        if not hasattr(entity, field):
            continue

        col = getattr(entity, field)
        coltype = getattr(col, "type", None)

        def _use_ilike(v) -> bool:
            if op_hint == "ilike":
                return True
            return is_text_type(coltype) and isinstance(v, str) and looks_like_wildcard(v)

        try:
            if isinstance(value, (list, tuple, set)):
                vals = [v for v in value if v not in (None, "", "null", "None")]
                if not vals:
                    continue
                if any(_use_ilike(v) for v in vals) and is_text_type(coltype):
                    patterns = [to_like_pattern(str(v)) for v in vals]
                    query = query.filter(or_(*[col.ilike(p) for p in patterns]))
                else:
                    coerced = [coerce_value_for_column(coltype, v) for v in vals]
                    if not coerced:
                        continue
                    query = query.filter(col.in_(coerced))
            else:
                if _use_ilike(value) and is_text_type(coltype):
                    query = query.filter(col.ilike(to_like_pattern(str(value))))
                else:
                    v = coerce_value_for_column(coltype, value)
                    query = query.filter(col.is_(None) if v is None else (col == v))
        except InvalidEnum as e:
            raise UnprocessableEntityException(str(e), loc=["query", "filters", raw_field]) from e

    return query



from __future__ import annotations
from typing import Optional
from uuid import uuid4

from starlette.datastructures import Headers
from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
from starlette.requests import Request

from nlbone import config
from nlbone.utils.context import request_ctx, request_id_ctx, bind_context, reset_context


def mock_request(user=None, token: Optional[str] = None):
    request = Request({
        "type": "http",
        "headers": Headers({"User-Agent": "Testing-Agent"}).raw,
        "client": {"host": "192.168.1.1", "port": 80},
        "method": "GET",
        "path": "/__test__",
    })
    request.state.user = user
    request.state.token = token
    return request


def current_request() -> Optional[Request]:
    req = request_ctx.get()
    if config.ENV == 'test' and req is None:
        return mock_request()
    return req


def current_request_id() -> Optional[str]:
    return request_id_ctx.get()


class AddRequestContextMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
        incoming_req_id = request.headers.get("X-Request-ID")
        req_id = incoming_req_id or str(uuid4())

        user_id = getattr(getattr(request, "state", None), "user_id", None) or request.headers.get("X-User-Id")
        ip = request.client.host if request.client else None
        ua = request.headers.get("user-agent")

        tokens = bind_context(request=request, request_id=req_id, user_id=user_id, ip=ip, user_agent=ua)
        try:
            response = await call_next(request)
            response.headers.setdefault("X-Request-ID", req_id)
            return response
        finally:
            reset_context(tokens)



from __future__ import annotations
from typing import Any, Optional, AsyncIterator
import httpx

from nlbone.core.ports.files import AsyncFileServicePort
from nlbone.config.settings import get_settings
from .uploadchi import UploadchiError, _auth_headers, _build_list_query, _filename_from_cd, _resolve_token

class UploadchiAsyncClient(AsyncFileServicePort):
    def __init__(self, base_url: Optional[str] = None, timeout_seconds: Optional[float] = None, client: httpx.AsyncClient | None = None) -> None:
        s = get_settings()
        self._base_url = base_url or str(s.UPLOADCHI_BASE_URL)
        self._timeout = timeout_seconds or float(s.HTTP_TIMEOUT_SECONDS)
        self._client = client or httpx.AsyncClient(base_url=self._base_url, timeout=self._timeout, follow_redirects=True)

    async def aclose(self) -> None:
        await self._client.aclose()

    async def upload_file(self, file_bytes: bytes, filename: str, params: dict[str, Any] | None = None, token: str | None = None) -> dict:
        tok = _resolve_token(token)
        files = {"file": (filename, file_bytes)}
        data = (params or {}).copy()
        r = await self._client.post("", files=files, data=data, headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, await r.aread())
        return r.json()

    async def commit_file(self, file_id: int, client_id: str, token: str | None = None) -> None:
        tok = _resolve_token(token)
        r = await self._client.post(f"/{file_id}/commit", headers=_auth_headers(tok), params={"client_id": client_id} if client_id else None)
        if r.status_code not in (204, 200):
            raise UploadchiError(r.status_code, await r.aread())

    async def list_files(self, limit: int = 10, offset: int = 0, filters: dict[str, Any] | None = None, sort: list[tuple[str, str]] | None = None, token: str | None = None) -> dict:
        tok = _resolve_token(token)
        q = _build_list_query(limit, offset, filters, sort)
        r = await self._client.get("", params=q, headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, await r.aread())
        return r.json()

    async def get_file(self, file_id: int, token: str | None = None) -> dict:
        tok = _resolve_token(token)
        r = await self._client.get(f"/{file_id}", headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, await r.aread())
        return r.json()

    async def download_file(self, file_id: int, token: str | None = None) -> tuple[AsyncIterator[bytes], str, str]:
        tok = _resolve_token(token)
        r = await self._client.get(f"/{file_id}/download", headers=_auth_headers(tok), stream=True)
        if r.status_code >= 400:
            body = await r.aread()
            raise UploadchiError(r.status_code, body.decode(errors="ignore"))
        filename = _filename_from_cd(r.headers.get("content-disposition"), fallback=f"file-{file_id}")
        media_type = r.headers.get("content-type", "application/octet-stream")

        async def _aiter() -> AsyncIterator[bytes]:
            try:
                async for chunk in r.aiter_bytes():
                    yield chunk
            finally:
                await r.aclose()

        return _aiter(), filename, media_type

    async def delete_file(self, file_id: int, token: str | None = None) -> None:
        tok = _resolve_token(token)
        r = await self._client.delete(f"/{file_id}", headers=_auth_headers(tok))
        if r.status_code not in (204, 200):
            body = await r.aread()
            raise UploadchiError(r.status_code, body.decode(errors="ignore"))



from __future__ import annotations
import json
from typing import Any, Optional
from urllib.parse import urlparse, urlunparse

import httpx
import requests

from nlbone.core.ports.files import FileServicePort
from nlbone.config.settings import get_settings


class UploadchiError(RuntimeError):
    def __init__(self, status: int, detail: Any | None = None):
        super().__init__(f"Uploadchi HTTP {status}: {detail}")
        self.status = status
        self.detail = detail


def _resolve_token(explicit: str | None) -> str | None:
    if explicit is not None:
        return explicit
    s = get_settings()
    return s.UPLOADCHI_TOKEN.get_secret_value() if s.UPLOADCHI_TOKEN else None


def _auth_headers(token: str | None) -> dict[str, str]:
    return {"Authorization": f"Bearer {token}"} if token else {}


def _build_list_query(limit: int, offset: int, filters: dict[str, Any] | None, sort: list[tuple[str, str]] | None) -> \
        dict[str, Any]:
    q: dict[str, Any] = {"limit": limit, "offset": offset}
    if filters:
        q["filters"] = json.dumps(filters)
    if sort:
        q["sort"] = ",".join([f"{f}:{o}" for f, o in sort])
    return q


def _filename_from_cd(cd: str | None, fallback: str) -> str:
    if not cd:
        return fallback
    if "filename=" in cd:
        return cd.split("filename=", 1)[1].strip("\"'")
    return fallback


def _normalize_https_base(url: str) -> str:
    p = urlparse(url.strip())
    p = p._replace(scheme="https")  # enforce https
    if p.path.endswith("/"):
        p = p._replace(path=p.path.rstrip("/"))
    return str(urlunparse(p))


class UploadchiClient(FileServicePort):
    def __init__(self, base_url: Optional[str] = None, timeout_seconds: Optional[float] = None,
                 client: httpx.Client | None = None) -> None:
        s = get_settings()
        self._base_url = _normalize_https_base(base_url or str(s.UPLOADCHI_BASE_URL))
        self._timeout = timeout_seconds or float(s.HTTP_TIMEOUT_SECONDS)
        self._client = client or requests.session()

    def close(self) -> None:
        self._client.close()

    def upload_file(self, file_bytes: bytes, filename: str, params: dict[str, Any] | None = None,
                    token: str | None = None) -> dict:
        tok = _resolve_token(token)
        files = {"file": (filename, file_bytes)}
        data = (params or {}).copy()
        r = self._client.post(self._base_url, files=files, data=data, headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, r.text)
        return r.json()

    def commit_file(self, file_id: int, client_id: str, token: str | None = None) -> None:
        tok = _resolve_token(token)
        r = self._client.post(f"{self._base_url}/{file_id}/commit", headers=_auth_headers(tok),
                              params={"client_id": client_id} if client_id else None)
        if r.status_code not in (204, 200):
            raise UploadchiError(r.status_code, r.text)

    def list_files(self, limit: int = 10, offset: int = 0, filters: dict[str, Any] | None = None,
                   sort: list[tuple[str, str]] | None = None, token: str | None = None) -> dict:
        tok = _resolve_token(token)
        q = _build_list_query(limit, offset, filters, sort)
        r = self._client.get(self._base_url, params=q, headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, r.text)
        return r.json()

    def get_file(self, file_id: int, token: str | None = None) -> dict:
        tok = _resolve_token(token)
        r = self._client.get(f"{self._base_url}/{file_id}", headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, r.text)
        return r.json()

    def download_file(self, file_id: int, token: str | None = None) -> tuple[bytes, str, str]:
        tok = _resolve_token(token)
        r = self._client.get(f"{self._base_url}/{file_id}/download", headers=_auth_headers(tok))
        if r.status_code >= 400:
            raise UploadchiError(r.status_code, r.text)
        filename = _filename_from_cd(r.headers.get("content-disposition"), fallback=f"file-{file_id}")
        media_type = r.headers.get("content-type", "application/octet-stream")
        return r.content, filename, media_type

    def delete_file(self, file_id: int, token: str | None = None) -> None:
        tok = _resolve_token(token)
        r = self._client.delete(f"{self._base_url}/{file_id}", headers=_auth_headers(tok))
        if r.status_code not in (204, 200):
            raise UploadchiError(r.status_code, r.text)



from __future__ import annotations
from typing import Any, Mapping, Optional

from dependency_injector import containers, providers

from nlbone.adapters.http_clients.uploadchi import UploadchiClient
from nlbone.adapters.http_clients.uploadchi_async import UploadchiAsyncClient
from nlbone.adapters.auth.keycloak import KeycloakAuthService
from nlbone.core.ports.files import FileServicePort, AsyncFileServicePort


class Container(containers.DeclarativeContainer):
    config = providers.Configuration(strict=False)

    auth: providers.Singleton[KeycloakAuthService] = providers.Singleton(KeycloakAuthService, settings=config)
    file_service: providers.Singleton[FileServicePort] = providers.Singleton(UploadchiClient)
    afiles_service: providers.Singleton[AsyncFileServicePort] = providers.Singleton(UploadchiAsyncClient)


def create_container(settings: Optional[Any] = None) -> Container:
    c = Container()
    if settings is not None:
        if hasattr(settings, "model_dump"):
            c.config.from_dict(settings.model_dump())  # Pydantic v2
        elif hasattr(settings, "dict"):
            c.config.from_dict(settings.dict())  # Pydantic v1
        elif isinstance(settings, Mapping):
            c.config.from_dict(dict(settings))
        else:
            c.config.override(settings)
    return c



from __future__ import annotations
from sqlalchemy.orm import Session, Query

from nlbone.adapters.db.sqlalchemy.query.filters import apply_filters_to_query
from nlbone.adapters.db.sqlalchemy.query.ordering import apply_order_to_query


def apply_pagination(pagination, entity, session: Session, limit: bool = True) -> Query:
    q: Query = session.query(entity)
    q = apply_filters_to_query(pagination, entity, q)
    from nlbone.adapters.db.sqlalchemy.query.ordering import apply_order_to_query
    q = apply_order_to_query(pagination, entity, q)
    if limit:
        q = q.limit(pagination.limit).offset(pagination.offset)
    return q

def apply_filters(pagination, entity, query: Query) -> Query:
    return apply_filters_to_query(pagination, entity, query)

def apply_order(pagination, entity, query: Query) -> Query:
    return apply_order_to_query(pagination, entity, query)



import os
from functools import lru_cache
from pathlib import Path
from typing import Literal
from pydantic import AnyHttpUrl, Field, SecretStr, AliasChoices
from pydantic_settings import BaseSettings, SettingsConfigDict


def _guess_env_file() -> str | None:
    explicit = os.getenv("NLBONE_ENV_FILE")
    if explicit:
        return explicit

    cwd_env = Path.cwd() / ".env"
    if cwd_env.exists():
        return str(cwd_env)

    for i in range(1, 8):
        p = Path(__file__).resolve().parents[i]
        f = p / ".env"
        if f.exists():
            return str(f)


class Settings(BaseSettings):
    # ---------------------------
    # App
    # ---------------------------
    ENV: Literal["local", "dev", "staging", "prod"] = Field(default="local")
    DEBUG: bool = Field(default=False)
    LOG_LEVEL: Literal["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"] = Field(default="INFO")
    LOG_JSON: bool = Field(default=True)

    # ---------------------------
    # HTTP / Timeouts
    # ---------------------------
    HTTP_TIMEOUT_SECONDS: float = Field(default=10.0)

    # ---------------------------
    # Keycloak / Auth
    # ---------------------------
    KEYCLOAK_SERVER_URL: AnyHttpUrl = Field(default="https://keycloak.local/auth",
                                            validation_alias=AliasChoices("NLBONE_KEYCLOAK_SERVER_URL",
                                                                          "KEYCLOAK_SERVER_URL"))
    KEYCLOAK_REALM_NAME: str = Field(default="numberland",
                                     validation_alias=AliasChoices("NLBONE_KEYCLOAK_REALM_NAME", "KEYCLOAK_REALM_NAME"))
    KEYCLOAK_CLIENT_ID: str = Field(default="nlbone",
                                    validation_alias=AliasChoices("NLBONE_KEYCLOAK_CLIENT_ID", "KEYCLOAK_CLIENT_ID"))
    KEYCLOAK_CLIENT_SECRET: SecretStr = Field(default=SecretStr("dev-secret"),
                                              validation_alias=AliasChoices("NLBONE_KEYCLOAK_CLIENT_SECRET",
                                                                            "KEYCLOAK_CLIENT_SECRET"))

    # ---------------------------
    # Database
    # ---------------------------
    POSTGRES_DB_DSN: str = Field(default="postgresql+asyncpg://user:pass@localhost:5432/nlbone")

    # ---------------------------
    # Messaging / Cache
    # ---------------------------
    REDIS_URL: str = Field(default="redis://localhost:6379/0")

    # ---------------------------
    # UPLOADCHI
    # ---------------------------
    UPLOADCHI_BASE_URL: AnyHttpUrl = Field(default="https://uploadchi.numberland.ir/v1/files")
    UPLOADCHI_TOKEN: SecretStr | None = Field(
        default=None,
        validation_alias=AliasChoices("NLBONE_UPLOADCHI_TOKEN", "UPLOADCHI_TOKEN"),
    )

    model_config = SettingsConfigDict(
        env_prefix="NLBONE_",
        env_file=None,
        env_file_encoding="utf-8",
        extra="ignore",
    )


@lru_cache(maxsize=4)
def get_settings(env_file: str | None = None) -> Settings:
    """
    Cached settings for fast access across the app.
    Usage:
        from nlbone.config.settings import get_settings
        settings = get_settings()
    """
    env_file = env_file or _guess_env_file()
    return Settings(_env_file=env_file)



from fastapi import HTTPException, status

class UnprocessableEntityException(HTTPException):
    def __init__(self, detail: str, loc: list[str] | None = None):
        super().__init__(
            status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
            detail={"msg": detail, "loc": loc or []},
        )



from typing import Protocol, runtime_checkable

@runtime_checkable
class AuthService(Protocol):
    def has_access(self, token: str, permissions: list[str]) -> bool: ...
    def verify_token(self, token: str) -> dict | None: ...
    def get_client_token(self) -> dict | None: ...
    def is_client_token(self, token: str, allowed_clients: set[str] | None = None) -> bool: ...
    def client_has_access(self, token: str, perms: list[str], allowed_clients: set[str] | None = None) -> bool: ...



from contextvars import ContextVar
from typing import Any, Optional

request_ctx: ContextVar[Any | None] = ContextVar("request", default=None)
request_id_ctx: ContextVar[str | None] = ContextVar("request_id", default=None)
user_id_ctx: ContextVar[str | None] = ContextVar("user_id", default=None)
ip_ctx: ContextVar[str | None] = ContextVar("ip", default=None)
user_agent_ctx: ContextVar[str | None] = ContextVar("user_agent", default=None)

def bind_context(*, request: Any | None = None, request_id: str | None = None,
                 user_id: str | None = None, ip: str | None = None,
                 user_agent: str | None = None) -> dict[ContextVar, Any]:
    tokens: dict[ContextVar, Any] = {}
    if request is not None:
        tokens[request_ctx] = request_ctx.set(request)
    if request_id is not None:
        tokens[request_id_ctx] = request_id_ctx.set(request_id)
    if user_id is not None:
        tokens[user_id_ctx] = user_id_ctx.set(user_id)
    if ip is not None:
        tokens[ip_ctx] = ip_ctx.set(ip)
    if user_agent is not None:
        tokens[user_agent_ctx] = user_agent_ctx.set(user_agent)
    return tokens

def reset_context(tokens: dict[ContextVar, Any]):
    for var, token in tokens.items():
        var.reset(token)

def current_request():
    return request_ctx.get()

def current_request_id() -> Optional[str]:
    return request_id_ctx.get()

def current_context_dict() -> dict[str, Any]:
    return {
        "request_id": request_id_ctx.get(),
        "user_id": user_id_ctx.get(),
        "ip": ip_ctx.get(),
        "user_agent": user_agent_ctx.get(),
    }


import argparse
import anyio

from nlbone.adapters.db.sqlalchemy.schema import init_db_async, init_db_sync

def main() -> None:
    parser = argparse.ArgumentParser(description="Initialize database schema (create_all).")
    parser.add_argument("--async", dest="use_async", action="store_true", help="Use AsyncEngine")
    args = parser.parse_args()

    if args.use_async:
        anyio.run(init_db_async)
    else:
        init_db_sync()

if __name__ == "__main__":
    main()




import importlib
from typing import Sequence

from nlbone.adapters.db.sqlalchemy.base import Base
from nlbone.adapters.db.sqlalchemy.engine import init_async_engine, init_sync_engine


DEFAULT_MODEL_MODULES: Sequence[str] = (
)

def import_model_modules(modules: Sequence[str] | None = None) -> None:
    for m in (modules or DEFAULT_MODEL_MODULES):
        importlib.import_module(m)

# --------- Async (SQLAlchemy 2.x) ----------
async def init_db_async(model_modules: Sequence[str] | None = None) -> None:
    """Create tables using AsyncEngine (dev/test). Prefer Alembic in prod."""
    import_model_modules(model_modules)
    engine = init_async_engine()
    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)

# --------- Sync ----------
def init_db_sync(model_modules: Sequence[str] | None = None) -> None:
    """Create tables using Sync Engine (dev/test). Prefer Alembic in prod."""
    import_model_modules(model_modules)
    engine = init_sync_engine()
    Base.metadata.create_all(bind=engine)